//
//  iTableView.m
//  iTableView
//
//  Created by b4j on 2/5/15.
//  Copyright (c) 2015 Anywhere Software. All rights reserved.
//

#import "iUI8.h"
@interface B4ITableSection : NSObject
@property (nonatomic)NSMutableArray *cells;
@property (nonatomic)NSString *header;
@property (nonatomic)NSString *footer;
@end
@interface B4ITableViewDelgate: NSObject <UITableViewDelegate, UITableViewDataSource>
@property (nonatomic) NSMutableArray *sections;
@end
@implementation B4ITableViewDelgate
- (instancetype)init {
    self = [super init];
    self.sections = [NSMutableArray new];
    return self;
}
- (B4ITableSection *)getSection:(int)i {
    return ((B4ITableSection *)self.sections[(NSUInteger) i]);
}
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return self.sections.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    B4ITableCell *tc = [self getSection:(int)indexPath.section].cells[(NSUInteger) indexPath.row];
    [B4IObjectWrapper raiseEvent:tableView :@"_selectedchanged::" :@[@(indexPath.section), tc]];
}
- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath {
    B4ITableCell *tc = [self getSection:(int)indexPath.section].cells[(NSUInteger) indexPath.row];
    [B4IObjectWrapper raiseEvent:tableView :@"_accessorybuttonclicked::" :@[@(indexPath.section), tc]];

}
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    int c = (int)[self getSection:(int)section].cells.count;
    return c;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    B4ITableSection *section = [self getSection:(int)indexPath.section];
    B4ITableCell *cell = section.cells[(NSUInteger) indexPath.row];
    UITableViewCell *uc = [tableView dequeueReusableCellWithIdentifier:cell.reusableIdentifier];
    uc = [cell createUI:uc];
    return uc;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)sec {
    B4ITableSection *section = [self getSection:(int)sec];
    NSString *s = section.header;
    if (s.Length > 0)
        return s;
    return nil;
}

- (NSString *)tableView:(UITableView *)tableView titleForFooterInSection:(NSInteger)sec {
    B4ITableSection *section = ((B4ITableSection *)self.sections[(NSUInteger) sec]);
    NSString *s = section.footer;
    if (s.Length > 0)
        return s;
    return nil;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
}


- (NSArray *)tableView:(UITableView *)tableView editActionsForRowAtIndexPath:(NSIndexPath *)indexPath {
    B4ITableSection *section = [self getSection:(int)indexPath.section];
    B4ITableCell *cell = section.cells[(NSUInteger) indexPath.row];
    if (cell.actionButtons == nil || cell.actionButtons.count == 0)
        return @[];
    NSMutableArray * items = [NSMutableArray new];
    for (int i = 0;i < cell.actionButtons.count;i += 2) {
        NSString *text = cell.actionButtons[(NSUInteger) i];
        int clr = [(NSNumber *) cell.actionButtons[(NSUInteger) (i + 1)] intValue];
        UITableViewRowAction* act = [UITableViewRowAction rowActionWithStyle:UITableViewRowActionStyleDefault
                                  title:text handler:^(UITableViewRowAction *action, NSIndexPath *indexPath2) {
                    [B4IObjectWrapper raiseEvent:tableView :@"_actionbuttonclicked:::" :@[@(indexPath.section), cell, text]];
                    tableView.editing = false;
                }];
        act.backgroundColor = [[B4I shared] ColorToUIColor:clr];
        [items addObject:act];
    }
    return items;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    B4ITableSection *section = [self getSection:(int)indexPath.section];
    B4ITableCell *cell = section.cells[(NSUInteger) indexPath.row];
    return (!(cell.actionButtons == nil || cell.actionButtons.count == 0));
}


@end

@implementation B4ITableSection

@end

@implementation B4ITableView {
}
#define OBJ ((UITableView *) self.object)
+(Class)getClass {
    return [UITableView class];
}

- (int)NumberOfSections {
    return (int)[self getSections].count;
}

- (int)SelectedSection {
    NSIndexPath *i = [OBJ indexPathForSelectedRow];
    if (i == nil)
        return -1;
    return (int)i.section;
}

- (int)SeparatorColor {
    return [[B4I shared] UIColorToColor:OBJ.separatorColor];
}

- (void)setSeparatorColor:(int)SeparatorColor {
    OBJ.separatorColor = [[B4I shared] ColorToUIColor:SeparatorColor];
}

- (float)RowHeight {
    return OBJ.rowHeight;
}

- (void)setRowHeight:(float)RowHeight {
    OBJ.rowHeight = RowHeight;
}

- (int)SCROLL_BOTTOM {
    return UITableViewScrollPositionBottom;
}




- (int)SCROLL_MIDDLE {
    return UITableViewScrollPositionMiddle;
}

- (int)SCROLL_TOP {
    return UITableViewScrollPositionTop;
}


- (B4ITableCell *)SelectedRow {
    NSIndexPath *i = [OBJ indexPathForSelectedRow];
    if (i == nil)
        return nil;
    return (B4ITableCell *) [[self GetItems:(int)i.section] Get:(int)i.row];
}

- (B4IList *)GetItems:(int)SectionIndex {
    B4ITableSection* section = [self getSections][(NSUInteger) SectionIndex];
    return [B4IObjectWrapper createWrapper:[B4IList new ] object:section.cells];
}
- (NSMutableArray *)getSections {
    return [(B4ITableViewDelgate *) OBJ.dataSource sections];
}

- (void)DesignerCreateView:(B4IPanelWrapper *)base :(B4ILabelWrapper *)lw :(B4IMap *)props {
    [self Initialize:[B4IObjectWrapper getBI:base.object] :[B4IObjectWrapper getEventName:base.object]
            :[(NSNumber *) [props Get:@"Grouped"] boolValue]];
    [base AddView:OBJ :0 :0 :base.Width :base.Height];

}

- (void)_initialize:(B4I *)bi2 :(NSObject *)target :(NSString *)EventName {
}

- (void)base_resize:(CGFloat)width :(CGFloat)height {
    [OBJ SetLayoutAnimated:0 :1 :0 :0 :width :height];
}
- (B4IViewWrapper *)BaseView {
    B4IWeakRef *ref = [B4IObjectWrapper getMap:OBJ][@"base"];
    if (ref == nil || ref.Value == nil)
        return self;
    return [B4IObjectWrapper createWrapper:[B4IPanelWrapper new] object:ref.Value];
}


- (void)Initialize:(B4I*)bi :(NSString *)EventName :(BOOL)Grouped{
    self.object = [[UITableView alloc] initWithFrame:CGRectZero style:Grouped ? UITableViewStyleGrouped : UITableViewStylePlain];
    [self innerInitialize:bi :EventName :true];
}
- (void)innerInitialize:(B4I *)bi :(NSString *)eventName :(BOOL)keepOldObject {
    if (!keepOldObject)
        self.object = [UITableView new];
    [super innerInitialize:bi :eventName :keepOldObject];
    B4ITableViewDelgate *del = [[B4ITableViewDelgate alloc] init];
    [B4IObjectWrapper getMap:OBJ][@"delegate"] = del;
    OBJ.delegate = del;
    OBJ.dataSource = del;
}
- (void)AddSection :(NSString *)Header :(NSString *)Footer {
    B4ITableSection *section = [B4ITableSection new];
    section.header = Header;
    section.footer = Footer;
    section.cells = [NSMutableArray new];
    [[(B4ITableViewDelgate *) OBJ.dataSource sections] addObject:section];

}
- (B4ITableCell *)AddSingleLine:(NSString *)Text {
    B4ITableCell *tc = [B4ITableCell new];
    [tc InitializeSingleLine:Text];
    return [self AddTableCell:tc];
}
- (B4ITableCell *)AddTwoLines:(NSString *)Text :(NSString *)DetailText {
    B4ITableCell *tc = [B4ITableCell new];
    [tc InitializeTwoLines:Text :DetailText];
    return [self AddTableCell:tc];
}
- (B4ITableCell *)AddTableCell:(B4ITableCell *) TableCell{
    NSMutableArray *sections = [(B4ITableViewDelgate *) OBJ.dataSource sections];
    if (sections.count == 0)
        [self AddSection:@"" :@""];
    B4ITableSection *sec = sections.lastObject;
    [sec.cells addObject:TableCell];
    return TableCell;
}
- (void)SetSelection:(int)SectionIndex :(int)RowIndex {
    if (RowIndex == -1) {
        [OBJ deselectRowAtIndexPath:[OBJ indexPathForSelectedRow] animated:true];
    } else {
        NSIndexPath *i = [NSIndexPath indexPathForRow:RowIndex inSection:SectionIndex];
        [OBJ selectRowAtIndexPath:i animated:true scrollPosition:UITableViewScrollPositionNone];
        [OBJ scrollToRowAtIndexPath:i atScrollPosition:UITableViewScrollPositionNone animated:true];
    }
}
- (void)Clear {
    [[(B4ITableViewDelgate *) OBJ.dataSource sections] removeAllObjects];
    [OBJ reloadData];
}
- (void)ReloadAll{
    [OBJ reloadData];
}
- (void)ReloadSection:(int)SectionIndex {
    NSIndexSet* set = [[NSIndexSet alloc] initWithIndex:(NSUInteger) SectionIndex];
    [OBJ reloadSections:set withRowAnimation:UITableViewRowAnimationAutomatic];
}
- (void)BeginUpdates {
    [OBJ beginUpdates];
}
- (void)InsertCells:(int)SectionIndex :(int)RowIndex :(B4IList *)Cells {
    [[self GetItems:0] AddAllAt:RowIndex :Cells];
    NSMutableArray *paths = [NSMutableArray new];
    for (int i = RowIndex;i < RowIndex + Cells.Size;i++)
        [paths addObject:[NSIndexPath indexPathForRow:i inSection:SectionIndex]];
    [OBJ insertRowsAtIndexPaths:paths withRowAnimation:UITableViewRowAnimationAutomatic];
}
- (void)RemoveCells:(int)SectionIndex :(int)StartRowIndex :(int)Count{
    B4ITableSection *sec = [self getSections][(NSUInteger) SectionIndex];
    NSMutableArray *paths = [NSMutableArray new];
    for (int i = StartRowIndex;i < StartRowIndex + Count;i++) {
        [paths addObject:[NSIndexPath indexPathForRow:i inSection:SectionIndex]];
    }
    [sec.cells removeObjectsInRange:NSMakeRange(StartRowIndex, Count)];
    [OBJ deleteRowsAtIndexPaths:paths withRowAnimation:UITableViewRowAnimationAutomatic];
}
- (void)EndUpdates {
    [OBJ endUpdates];
}
- (void)ScrollTo:(int)SectionIndex :(int)RowIndex :(int)ScrollPosition {
    [OBJ scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:RowIndex inSection:SectionIndex]
               atScrollPosition:(UITableViewScrollPosition) ScrollPosition animated:true];
}



@end

@implementation B4ITableCell

static NSString* B4ICellStyleDefault = @"default";
static NSString* B4ICellStyleSubtitle = @"subtitle";
- (instancetype)init {
    self = [super init];
    self.DetailText = [B4IAttributedString new];
    self.Text = [B4IAttributedString new];
    self.Bitmap = [B4IBitmap new];
    self.IndentationWidth = 10.0f;
    self.ShowSelection = true;
    return self;
}
- (UITableViewCell *)createUI:(UITableViewCell *)current {
    if (current == nil) {
        current = [[UITableViewCell alloc] initWithStyle:self.cellStyle reuseIdentifier:self.reusableIdentifier];
    }
    current.textLabel.attributedText = self.Text.objectOrNull;
    current.detailTextLabel.attributedText = self.DetailText.objectOrNull;
    current.imageView.image = self.Bitmap.objectOrNull;
    current.accessoryType = (UITableViewCellAccessoryType) self.AccessoryType;
    current.indentationLevel = self.IndentationLevel;
    current.indentationWidth = self.IndentationWidth;
    current.selectionStyle = self.ShowSelection ? UITableViewCellSelectionStyleDefault : UITableViewCellSelectionStyleNone;
    B4ITableCell *prev = [B4IObjectWrapper getMap:current][@"data"];
    if (prev != nil && prev.CustomView != nil) {
        for (UIView *v in current.contentView.subviews) {
            if ([v isKindOfClass:[B4IPanelView class]])
                [v removeFromSuperview];
        }
    }
    if (self.CustomView != nil && self.CustomView.IsInitialized)
        [current.contentView addSubview:self.CustomView.object];
    [B4IObjectWrapper getMap:current][@"data"] = self;
    self.nativeCell = current;
    return current;
}



- (void)InitializeSingleLine:(NSString *)Text {
    self.reusableIdentifier = B4ICellStyleDefault;
    self.Text.object = [[NSAttributedString alloc] initWithString:Text];
    self.cellStyle = UITableViewCellStyleDefault;
}
- (void)InitializeTwoLines:(NSString *)Text :(NSString *)DetailText {
    self.reusableIdentifier = B4ICellStyleSubtitle;
    self.Text.object = [[NSAttributedString alloc] initWithString:Text];
    self.DetailText.object = [[NSAttributedString alloc] initWithString:DetailText];
    self.cellStyle = UITableViewCellStyleSubtitle;
}

- (int)ACCESSORY_NONE {
    return UITableViewCellAccessoryNone;
}

- (int)ACCESSORY_INDICATOR {
    return UITableViewCellAccessoryDisclosureIndicator;
}

- (int)ACCESSORY_DISCLOSURE_BUTTON {
    return UITableViewCellAccessoryDetailDisclosureButton;
}

- (int)ACCESSORY_DETAIL_BUTTON {
    return UITableViewCellAccessoryDetailButton;
}

- (int)ACCESSORY_CHECKMARK {
    return UITableViewCellAccessoryCheckmark;
}


- (void)AddActionButton:(NSString *)Text :(int)Color {
    if (self.actionButtons == nil)
        self.actionButtons = [NSMutableArray new];
    [self.actionButtons addObject:Text];
    [self.actionButtons addObject:@(Color)];
}

- (void)ClearActionButtons {
    self.actionButtons = nil;
}

- (void)Update {
    UITableViewCell *native = self.nativeCell;
    if (native != nil && [B4IObjectWrapper getMap:native][@"data"] == self)
        [self createUI:native];
}
@end